Exception: PaperTrailDiff::AmbiguousVersionOrderError
- Defined in:
- lib/paper_trail_diff/errors.rb,
sig/generated/paper_trail_diff/errors.rbs
Overview
Raised when versions sharing a timestamp cannot be ordered by their ids.
Raised when versions sharing a timestamp cannot be ordered by their ids.