Exception: PaperTrailDiff::VersionMismatchError
- Defined in:
- lib/paper_trail_diff/errors.rb,
sig/generated/paper_trail_diff/errors.rbs
Overview
Raised when compare endpoints do not belong to the same PaperTrail item.
Raised when compare endpoints do not belong to the same PaperTrail item.